@@ -65,6 +65,8 @@ typedef struct {
 	int   border_width;
 	unsigned long  border_color;
 	int   test_pattern;
+        int   is_override_redirect;
+        int   ignore_damage;
 } MagnifierOptions;
 
 static MagnifierOptions global_options = { ":0.0",
@@ -83,8 +85,8 @@ static MagnifierOptions global_options = { ":0.0",
 					   0,
 					   0,
 					   0,
-					   0,					   
-					   0,
+					   0,	
+                                   	   0,
 					   0,
 					   0,
 					   10,
@@ -93,6 +95,8 @@ static MagnifierOptions global_options = { ":0.0",
 					   1,
 					   0,
 					   0,
+					   0,
+					   0,
 					   0
                                          };
 
@@ -122,6 +126,8 @@ struct poptOption magnifier_options [] = {
 	{"border-size", 'b', POPT_ARG_INT, &global_options.border_width, '\0', "width of border", NULL},
 	{"border-color", 'c', POPT_ARG_LONG, &global_options.border_color, '\0', "border color specified as (A)RGB 23-bit value, Alpha-MSB", NULL},
 	{"use-test-pattern", '\0', POPT_ARG_NONE, &global_options.test_pattern, '\0', "use test pattern as source", NULL},
+	{"override-redirect", '\0', POPT_ARG_NONE, &global_options.is_override_redirect, '\0', "make the magnifier window totally unmanaged by the window manager", NULL},
+	{"ignore-damage", '\0', POPT_ARG_NONE, &global_options.ignore_damage, '\0', "ignore the X server DAMAGE extension, if present", NULL},
 	POPT_AUTOHELP
 	{NULL, 0, 0, NULL, 0, 0}
 };
@@ -306,15 +312,17 @@ magnifier_main_refresh_all (gpointer data)
 	regions = GNOME_Magnifier_Magnifier_getZoomRegions (
 		BONOBO_OBJREF (magnifier),
 		&ev);
-#ifdef DEBUG
+
+#ifdef DEBUG_REFRESH
 	fprintf (stderr, "refreshing %d regions\n", regions->_length);
 #endif
+#undef DEBUG_REFRESH
 
 	properties = GNOME_Magnifier_Magnifier_getProperties (BONOBO_OBJREF (magnifier), &ev);
 
-	dirty_bounds_any = Bonobo_PropertyBag_getValue (properties, "source-display-size", &ev);
+	dirty_bounds_any = Bonobo_PropertyBag_getValue (properties, "source-display-bounds", &ev);
 	if (BONOBO_EX (&ev)) {
-		g_warning ("Error getting source-display-size");
+		g_warning ("Error getting source-display-bounds");
 		bonobo_main_quit ();
 		return FALSE;
 	}
@@ -340,7 +348,7 @@ main (int argc, char** argv)
   GNOME_Magnifier_RectBounds *roi = GNOME_Magnifier_RectBounds__alloc();
   GNOME_Magnifier_RectBounds *viewport = GNOME_Magnifier_RectBounds__alloc();
   CORBA_any *viewport_any;
-  int x = 0, y = 0;
+  int x = 0, y = 0, fullwidth, fullheight;
   guint pan_handle = 0, refresh_handle = 0;
   CORBA_Environment ev;
   Bonobo_PropertyBag properties;
@@ -380,7 +388,12 @@ main (int argc, char** argv)
   /* FIXME */
   gtk_init (&argc, &argv);
 
-  magnifier = magnifier_new ();
+  if (global_options.ignore_damage)
+  {
+      g_setenv ("MAGNIFIER_IGNORE_DAMAGE", "1", TRUE);
+  }
+
+  magnifier = magnifier_new (global_options.is_override_redirect);
   fprintf (stderr, "New magnifier created.\n");
   
   properties = GNOME_Magnifier_Magnifier_getProperties (
@@ -412,10 +425,10 @@ main (int argc, char** argv)
 				     global_options.cursor_color, 
 				     NULL);
 
-  screen_width = gdk_screen_get_width (
+  fullwidth = screen_width = gdk_screen_get_width (
 	  gdk_display_get_screen (magnifier->target_display,
 				  magnifier->target_screen_num));
-  screen_height = gdk_screen_get_height (
+  fullheight = screen_height = gdk_screen_get_height (
 	  gdk_display_get_screen (magnifier->target_display,
 				  magnifier->target_screen_num));
 
@@ -430,15 +443,24 @@ main (int argc, char** argv)
   fprintf (stderr, "initial viewport %d %d\n", (int) screen_width,
 	   (int) screen_height);
 
-  viewport_any = CORBA_any__alloc ();
   init_rect_bounds (viewport, x, y, x + screen_width, y + screen_height);
-  viewport_any->_type = TC_GNOME_Magnifier_RectBounds;
-  viewport_any->_value = ORBit_copy_value (viewport,
-					   TC_GNOME_Magnifier_RectBounds);
+  viewport_any = bonobo_arg_new_from (TC_GNOME_Magnifier_RectBounds, viewport);
   
-  Bonobo_PropertyBag_setValue (properties, "target-display-bounds",
-			       viewport_any,
-			       &ev);
+  bonobo_pbclient_set_value (properties, "target-display-bounds",
+			     viewport_any,
+			     &ev);
+  bonobo_arg_release (viewport_any);
+
+  if (global_options.vertical_split || global_options.horizontal_split) 
+  {
+      init_rect_bounds (viewport, 0, 0, fullwidth-x, fullheight-y);
+      viewport_any = bonobo_arg_new_from (TC_GNOME_Magnifier_RectBounds, viewport);
+      bonobo_pbclient_set_value (properties, "source-display-bounds",
+				 viewport_any,
+				 &ev);
+
+      bonobo_arg_release (viewport_any);
+  }
 
   bonobo_object_release_unref (properties, NULL);
   properties = NULL;
@@ -449,7 +471,7 @@ main (int argc, char** argv)
   {
 	  int scroll_policy;
 	  
-	  fprintf (stderr, "creating an initial zoom region.\n");
+	  fprintf (stderr, "Creating an initial zoom region.\n");
 	  init_rect_bounds (roi, 0, 0, 100, 100);
 	  init_rect_bounds (viewport, 0, 0, screen_width, screen_height);
 	  zoom_region =
@@ -475,12 +497,12 @@ main (int argc, char** argv)
 				       global_options.timing_output, &ev);
 	  bonobo_pbclient_set_long (properties, "timing-pan-rate",
 				       global_options.timing_pan_rate, &ev);
-	  bonobo_pbclient_set_boolean (properties, "smooth-scroll-policy",
-				       scroll_policy, &ev);
-	  bonobo_pbclient_set_long    (properties, "border_size",
+	  bonobo_pbclient_set_long    (properties, "border-size",
 				       global_options.border_width, &ev);
 	  bonobo_pbclient_set_long    (properties, "border-color",
 				       global_options.border_color, &ev);
+	  bonobo_pbclient_set_short   (properties, "smooth-scroll-policy",
+				       (short) scroll_policy, &ev);
 	  bonobo_pbclient_set_boolean (properties, "use-test-pattern",
 				       global_options.test_pattern, &ev);
 
@@ -527,9 +549,13 @@ main (int argc, char** argv)
   }
   else
   {
-	  refresh_handle = g_timeout_add (
-		  global_options.refresh_time,
-		  magnifier_main_refresh_all, magnifier);
+	  if (global_options.ignore_damage ||
+	      !magnifier_source_has_damage_extension (magnifier)) 
+	  {
+		  refresh_handle = g_timeout_add (
+			  global_options.refresh_time,
+			  magnifier_main_refresh_all, magnifier);
+	  }
 	  
 	  pan_handle = g_timeout_add (
 		  global_options.mouse_poll_time,
